/*
 * Copyright 2007 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.core;

import java.util.Arrays;

import org.wamblee.reflection.ReflectionUtils;

public class DefaultRequiredInterface implements RequiredInterface {

	private String _name;
	private boolean _optional; 
	private Class[] _required;
	private ProvidedInterface _provider; 
	
	public DefaultRequiredInterface(String aName, Class aInterface) {
		this(aName, new Class[] { aInterface }); 
	}

	public DefaultRequiredInterface(String aName, Class[] aInterfaces) {
		this(aName, aInterfaces, false); 
	}
	
	public DefaultRequiredInterface(String aName, Class aInterface, boolean aIsOptional) {
	    this(aName, new Class[] { aInterface }, aIsOptional );
	}

	
	public DefaultRequiredInterface(String aName, Class[] aInterfaces, boolean aIsOptional) {
		_name = aName; 
		_optional = aIsOptional; 
		_required = aInterfaces; 
	}

	@Override
	public String getName() {
		return _name;
	}
	
	@Override
	public boolean isOptional() {
		return _optional; 
	}
	
	@Override
	public boolean implementedBy(ProvidedInterface aDescriptor) {
		Class[] provided = aDescriptor.getInterfaceTypes();
		for (Class required : _required) {
			if ( !serviceProvided(required, provided)) { 
				return false; 
			}
		}
		// all required interfaces are provided.
		return true; 
	}
	
	/**
	 * Check if the required interface is implemented by one of the provided interfaces. 
	 * @param aRequired required interface
	 * @param aProvided Provided interfaces.
	 * @return
	 */
	private boolean serviceProvided(Class aRequired, Class[] aProvided) { 
		for (Class provided: aProvided) {
			try {
				provided = ReflectionUtils.wrapIfNeeded(provided);
				aRequired = ReflectionUtils.wrapIfNeeded(aRequired);
				provided.asSubclass(aRequired);
				return true; 
			} catch (ClassCastException e) {
				// No match, try the next one.
			}
		}
		return false; 
	}
	
	@Override
	public ProvidedInterface getProvider() {
		return _provider; 
	}
	
	@Override
	public void setProvider(ProvidedInterface aProvider) {
	    assert aProvider != null; 
	    assert implementedBy(aProvider);
		_provider = aProvider; 	
	}
	
	@Override
	public boolean equals(Object obj) { 
	    return this == obj; 
	}
	
	@Override
	public boolean covers(RequiredInterface obj) {
	    // TODO do more than equals. 
		if ( !(obj instanceof DefaultRequiredInterface)) { 
			return false; 
		}
		DefaultRequiredInterface descr = (DefaultRequiredInterface)obj;
		if ( _required.length != descr._required.length ) { 
			return false; 
		}
		String[] interfaces1 = new String[_required.length];
		String[] interfaces2 = new String[_required.length];
		for (int i = 0; i < _required.length; i++) {  
			interfaces1[i] = _required[i].getName();
			interfaces2[i] = descr._required[i].getName();
		}
		Arrays.sort(interfaces1);
		Arrays.sort(interfaces2);
		return Arrays.equals(interfaces1, interfaces2);
	}

	@Override
	public int hashCode() {
		return _required.hashCode(); 
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("." + getName() + ":");
		for (Class intf: _required) { 
			buf.append("." + intf.getName());
		}
		return buf.toString();
	}
}
