/*
 * Copyright 2008 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */ 
package org.wamblee.system.graph.component;

import java.util.List;

import org.wamblee.system.core.SystemAssemblyException;
import org.wamblee.system.graph.Edge;
import org.wamblee.system.graph.Graph;
import org.wamblee.system.graph.Node;
import org.wamblee.system.graph.Visitor;

/**
 * Visitor that checks whether all externally provided interfaces are actually provided 
 * by any of the internal components. 
 * 
 * @author Erik Brakkee
 *
 */
public class CheckExternallyProvidedVisitor implements Visitor {
    
    private Graph _graph; 

    /**
     * Constructs the visitor. 
     * @param aGraph Component graph. 
     */
    public CheckExternallyProvidedVisitor(Graph aGraph) { 
        _graph = aGraph; 
    }

    @Override
    public void visitEdge(Edge aEdge) {
        // Empty.
    }

    @Override
    public void visitNode(Node aNode) {
        if ( aNode instanceof ExternalProvidedInterfaceNode) { 
            ExternalProvidedInterfaceNode provided = (ExternalProvidedInterfaceNode) aNode; 
            List<Edge> edges = _graph.findOutgoing(provided);
            if ( edges.size() > 2 ) { 
                createDuplicateException("External provided interfaces has multiple internal matches", aNode, edges);
            }
            if ( edges.size() == 0 ) { 
                throw new SystemAssemblyException(aNode + ": external provded interface is not provided by any of the internal components");
            }
        }
    }
    
    private void createDuplicateException(String aMsg, Node aNode, List<Edge> edges) {
        StringBuffer buf = new StringBuffer();
        buf.append(aNode
                + ": " + aMsg + ": ");
        for (Edge edge : edges) {
            buf.append(edge.getTo() + "/ ");
        }
        throw new SystemAssemblyException(buf.toString());
    }

}
