/*
 * Copyright 2008 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wamblee.system.graph.component;

import java.util.List;

import org.wamblee.system.core.SystemAssemblyException;
import org.wamblee.system.graph.Edge;
import org.wamblee.system.graph.Graph;
import org.wamblee.system.graph.Node;
import org.wamblee.system.graph.Visitor;

/**
 * Visitor that checks whether required and provided interfaces are matched
 * appropriately:
 * <ul>
 * <li>Each required interface is connected to at most one provided interface
 * </li>
 * <li>Required interfaces that are not optional must be connected to precisely
 * one provided interface</li>
 * </ul>
 * 
 * @author Erik Brakkee
 * 
 */
public class CheckRequiredProvidedMultiplicityVisitor implements Visitor {

    private Graph _graph;

    public CheckRequiredProvidedMultiplicityVisitor(Graph aGraph) {
        _graph = aGraph;
    }

    @Override
    public void visitEdge(Edge aEdge) {
        // Empty
    }

    @Override
    public void visitNode(Node aNode) {
        if (aNode instanceof RequiredInterfaceNode) {
            RequiredInterfaceNode required = (RequiredInterfaceNode) aNode;
            List<Edge> edges = _graph.findOutgoing(aNode);
            if (edges.size() > 1) {
                createDuplicateException("Multiple providers of required interface found", aNode, edges);
            }
            if (edges.size() == 0 && !required.getRequired().isOptional()) {
                throw new SystemAssemblyException(
                        aNode
                                + ": mandatpory required interface not provided by other components started earlier");
            }
        } else if ( aNode instanceof ExternalProvidedInterfaceNode) { 
            List<Edge> edges = _graph.findOutgoing(aNode); 
            if ( edges.size() > 1) { 
                createDuplicateException("multiple internal matches for externally provided interface", aNode, edges);
            }
            if ( edges.size() == 0 ) { 
                throw new SystemAssemblyException(aNode + ": external provided interface is not provided internally");
            }
        }
    }

    private void createDuplicateException(String aMsg, Node aNode, List<Edge> edges) {
        StringBuffer buf = new StringBuffer();
        buf.append(aNode
                + ": " + aMsg + ": ");
        for (Edge edge : edges) {
            buf.append(edge.getTo() + "/ ");
        }
        throw new SystemAssemblyException(buf.toString());
    }

}
